@@ -20,7 +20,7 @@
 #
 import logging
 import random
-from typing import TYPE_CHECKING, Dict, Iterable, List, Optional, Set, Tuple
+from typing import TYPE_CHECKING, Iterable
 
 import attr
 
@@ -28,7 +28,6 @@ from synapse.api.constants import EduTypes
 from synapse.api.errors import AuthError, ShadowBanError, SynapseError
 from synapse.appservice import ApplicationService
 from synapse.metrics.background_process_metrics import (
-    run_as_background_process,
     wrap_as_background_process,
 )
 from synapse.replication.tcp.streams import TypingStream
@@ -42,6 +41,7 @@ from synapse.types import (
     UserID,
 )
 from synapse.util.caches.stream_change_cache import StreamChangeCache
+from synapse.util.duration import Duration
 from synapse.util.metrics import Measure
 from synapse.util.retryutils import filter_destinations_by_retry_limiter
 from synapse.util.wheel_timer import WheelTimer
@@ -61,15 +61,15 @@ class RoomMember:
 
 
 # How often we expect remote servers to resend us presence.
-FEDERATION_TIMEOUT = 60 * 1000
+FEDERATION_TIMEOUT = Duration(minutes=1)
 
 # How often to resend typing across federation.
-FEDERATION_PING_INTERVAL = 40 * 1000
+FEDERATION_PING_INTERVAL = Duration(seconds=40)
 
 
 # How long to remember a typing notification happened in a room before
 # forgetting about it.
-FORGET_TIMEOUT = 10 * 60 * 1000
+FORGET_TIMEOUT = Duration(minutes=10)
 
 
 class FollowerTypingHandler:
@@ -78,11 +78,10 @@ class FollowerTypingHandler:
     """
 
     def __init__(self, hs: "HomeServer"):
+        self.hs = hs  # nb must be called this for @wrap_as_background_process
         self.store = hs.get_datastores().main
         self._storage_controllers = hs.get_storage_controllers()
-        self.server_name = (
-            hs.hostname
-        )  # nb must be called this for @wrap_as_background_process
+        self.server_name = hs.hostname
         self.clock = hs.get_clock()
         self.is_mine_id = hs.is_mine_id
         self.is_mine_server_name = hs.is_mine_server_name
@@ -98,17 +97,17 @@ class FollowerTypingHandler:
             )
 
         # map room IDs to serial numbers
-        self._room_serials: Dict[str, int] = {}
+        self._room_serials: dict[str, int] = {}
         # map room IDs to sets of users currently typing
-        self._room_typing: Dict[str, Set[str]] = {}
+        self._room_typing: dict[str, set[str]] = {}
 
-        self._member_last_federation_poke: Dict[RoomMember, int] = {}
+        self._member_last_federation_poke: dict[RoomMember, int] = {}
         self.wheel_timer: WheelTimer[RoomMember] = WheelTimer(bucket_size=5000)
         self._latest_room_serial = 0
 
-        self._rooms_updated: Set[str] = set()
+        self._rooms_updated: set[str] = set()
 
-        self.clock.looping_call(self._handle_timeouts, 5000)
+        self.clock.looping_call(self._handle_timeouts, Duration(seconds=5))
         self.clock.looping_call(self._prune_old_typing, FORGET_TIMEOUT)
 
     def _reset(self) -> None:
@@ -143,10 +142,12 @@ class FollowerTypingHandler:
         # user.
         if self.federation and self.is_mine_id(member.user_id):
             last_fed_poke = self._member_last_federation_poke.get(member, None)
-            if not last_fed_poke or last_fed_poke + FEDERATION_PING_INTERVAL <= now:
-                run_as_background_process(
+            if (
+                not last_fed_poke
+                or last_fed_poke + FEDERATION_PING_INTERVAL.as_millis() <= now
+            ):
+                self.hs.run_as_background_process(
                     "typing._push_remote",
-                    self.server_name,
                     self._push_remote,
                     member=member,
                     typing=True,
@@ -168,7 +169,7 @@ class FollowerTypingHandler:
 
             now = self.clock.time_msec()
             self.wheel_timer.insert(
-                now=now, obj=member, then=now + FEDERATION_PING_INTERVAL
+                now=now, obj=member, then=now + FEDERATION_PING_INTERVAL.as_millis()
             )
 
             hosts: StrCollection = (
@@ -198,7 +199,7 @@ class FollowerTypingHandler:
             logger.exception("Error pushing typing notif to remotes")
 
     def process_replication_rows(
-        self, token: int, rows: List[TypingStream.TypingStreamRow]
+        self, token: int, rows: list[TypingStream.TypingStreamRow]
     ) -> None:
         """Should be called whenever we receive updates for typing stream."""
 
@@ -220,9 +221,8 @@ class FollowerTypingHandler:
             self._rooms_updated.add(row.room_id)
 
             if self.federation:
-                run_as_background_process(
+                self.hs.run_as_background_process(
                     "_send_changes_in_typing_to_remotes",
-                    self.server_name,
                     self._send_changes_in_typing_to_remotes,
                     row.room_id,
                     prev_typing,
@@ -230,7 +230,7 @@ class FollowerTypingHandler:
                 )
 
     async def _send_changes_in_typing_to_remotes(
-        self, room_id: str, prev_typing: Set[str], now_typing: Set[str]
+        self, room_id: str, prev_typing: set[str], now_typing: set[str]
     ) -> None:
         """Process a change in typing of a room from replication, sending EDUs
         for any local users.
@@ -284,7 +284,7 @@ class TypingWriterHandler(FollowerTypingHandler):
         hs.get_distributor().observe("user_left_room", self.user_left_room)
 
         # clock time we expect to stop
-        self._member_typing_until: Dict[RoomMember, int] = {}
+        self._member_typing_until: dict[RoomMember, int] = {}
 
         # caches which room_ids changed at which serials
         self._typing_stream_change_cache = StreamChangeCache(
@@ -319,7 +319,7 @@ class TypingWriterHandler(FollowerTypingHandler):
 
         if requester.shadow_banned:
             # We randomly sleep a bit just to annoy the requester.
-            await self.clock.sleep(random.randint(1, 10))
+            await self.clock.sleep(Duration(seconds=random.randint(1, 10)))
             raise ShadowBanError()
 
         await self.auth.check_user_in_room(room_id, requester)
@@ -354,7 +354,7 @@ class TypingWriterHandler(FollowerTypingHandler):
 
         if requester.shadow_banned:
             # We randomly sleep a bit just to annoy the requester.
-            await self.clock.sleep(random.randint(1, 10))
+            await self.clock.sleep(Duration(seconds=random.randint(1, 10)))
             raise ShadowBanError()
 
         await self.auth.check_user_in_room(room_id, requester)
@@ -384,9 +384,8 @@ class TypingWriterHandler(FollowerTypingHandler):
     def _push_update(self, member: RoomMember, typing: bool) -> None:
         if self.hs.is_mine_id(member.user_id):
             # Only send updates for changes to our own users.
-            run_as_background_process(
+            self.hs.run_as_background_process(
                 "typing._push_remote",
-                self.server_name,
                 self._push_remote,
                 member,
                 typing,
@@ -433,8 +432,10 @@ class TypingWriterHandler(FollowerTypingHandler):
         if user.domain in domains:
             logger.info("Got typing update from %s: %r", user_id, content)
             now = self.clock.time_msec()
-            self._member_typing_until[member] = now + FEDERATION_TIMEOUT
-            self.wheel_timer.insert(now=now, obj=member, then=now + FEDERATION_TIMEOUT)
+            self._member_typing_until[member] = now + FEDERATION_TIMEOUT.as_millis()
+            self.wheel_timer.insert(
+                now=now, obj=member, then=now + FEDERATION_TIMEOUT.as_millis()
+            )
             self._push_update_local(member=member, typing=content["typing"])
 
     def _push_update_local(self, member: RoomMember, typing: bool) -> None:
@@ -457,7 +458,7 @@ class TypingWriterHandler(FollowerTypingHandler):
 
     async def get_all_typing_updates(
         self, instance_name: str, last_id: int, current_id: int, limit: int
-    ) -> Tuple[List[Tuple[int, list]], int, bool]:
+    ) -> tuple[list[tuple[int, list]], int, bool]:
         """Get updates for typing replication stream.
 
         Args:
@@ -509,7 +510,7 @@ class TypingWriterHandler(FollowerTypingHandler):
         return rows, current_id, limited
 
     def process_replication_rows(
-        self, token: int, rows: List[TypingStream.TypingStreamRow]
+        self, token: int, rows: list[TypingStream.TypingStreamRow]
     ) -> None:
         # The writing process should never get updates from replication.
         raise Exception("Typing writer instance got typing info over replication")
@@ -536,7 +537,7 @@ class TypingNotificationEventSource(EventSource[int, JsonMapping]):
 
     async def get_new_events_as(
         self, from_key: int, service: ApplicationService
-    ) -> Tuple[List[JsonMapping], int]:
+    ) -> tuple[list[JsonMapping], int]:
         """Returns a set of new typing events that an appservice
         may be interested in.
 
@@ -581,9 +582,9 @@ class TypingNotificationEventSource(EventSource[int, JsonMapping]):
         limit: int,
         room_ids: Iterable[str],
         is_guest: bool,
-        explicit_room_id: Optional[str] = None,
-        to_key: Optional[int] = None,
-    ) -> Tuple[List[JsonMapping], int]:
+        explicit_room_id: str | None = None,
+        to_key: int | None = None,
+    ) -> tuple[list[JsonMapping], int]:
         """
         Find typing notifications for given rooms (> `from_token` and <= `to_token`)
         """
