@@ -29,12 +29,8 @@ from typing import (
     TYPE_CHECKING,
     Awaitable,
     BinaryIO,
-    Dict,
     Generator,
-    List,
     Optional,
-    Tuple,
-    Type,
 )
 
 import attr
@@ -50,6 +46,7 @@ from synapse.api.errors import Codes, cs_error
 from synapse.http.server import finish_request, respond_with_json
 from synapse.http.site import SynapseRequest
 from synapse.logging.context import (
+    PreserveLoggingContext,
     defer_to_threadpool,
     make_deferred_yieldable,
     run_in_background,
@@ -137,8 +134,8 @@ async def respond_with_file(
     request: SynapseRequest,
     media_type: str,
     file_path: str,
-    file_size: Optional[int] = None,
-    upload_name: Optional[str] = None,
+    file_size: int | None = None,
+    upload_name: str | None = None,
 ) -> None:
     logger.debug("Responding with %r", file_path)
 
@@ -160,8 +157,8 @@ async def respond_with_file(
 def add_file_headers(
     request: Request,
     media_type: str,
-    file_size: Optional[int],
-    upload_name: Optional[str],
+    file_size: int | None,
+    upload_name: str | None,
 ) -> None:
     """Adds the correct response headers in preparation for responding with the
     media.
@@ -305,10 +302,10 @@ def _can_encode_filename_as_token(x: str) -> bool:
 async def respond_with_multipart_responder(
     clock: Clock,
     request: SynapseRequest,
-    responder: "Optional[Responder]",
+    responder: "Responder | None",
     media_type: str,
-    media_length: Optional[int],
-    upload_name: Optional[str],
+    media_length: int | None,
+    upload_name: str | None,
 ) -> None:
     """
     Responds to requests originating from the federation media `/download` endpoint by
@@ -396,10 +393,10 @@ async def respond_with_multipart_responder(
 
 async def respond_with_responder(
     request: SynapseRequest,
-    responder: "Optional[Responder]",
+    responder: "Responder | None",
     media_type: str,
-    file_size: Optional[int],
-    upload_name: Optional[str] = None,
+    file_size: int | None,
+    upload_name: str | None = None,
 ) -> None:
     """Responds to the request with given responder. If responder is None then
     returns 404.
@@ -505,9 +502,9 @@ class Responder(ABC):
 
     def __exit__(  # noqa: B027
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         pass
 
@@ -530,47 +527,47 @@ class FileInfo:
     """Details about a requested/uploaded file."""
 
     # The server name where the media originated from, or None if local.
-    server_name: Optional[str]
+    server_name: str | None
     # The local ID of the file. For local files this is the same as the media_id
     file_id: str
     # If the file is for the url preview cache
     url_cache: bool = False
     # Whether the file is a thumbnail or not.
-    thumbnail: Optional[ThumbnailInfo] = None
+    thumbnail: ThumbnailInfo | None = None
 
     # The below properties exist to maintain compatibility with third-party modules.
     @property
-    def thumbnail_width(self) -> Optional[int]:
+    def thumbnail_width(self) -> int | None:
         if not self.thumbnail:
             return None
         return self.thumbnail.width
 
     @property
-    def thumbnail_height(self) -> Optional[int]:
+    def thumbnail_height(self) -> int | None:
         if not self.thumbnail:
             return None
         return self.thumbnail.height
 
     @property
-    def thumbnail_method(self) -> Optional[str]:
+    def thumbnail_method(self) -> str | None:
         if not self.thumbnail:
             return None
         return self.thumbnail.method
 
     @property
-    def thumbnail_type(self) -> Optional[str]:
+    def thumbnail_type(self) -> str | None:
         if not self.thumbnail:
             return None
         return self.thumbnail.type
 
     @property
-    def thumbnail_length(self) -> Optional[int]:
+    def thumbnail_length(self) -> int | None:
         if not self.thumbnail:
             return None
         return self.thumbnail.length
 
 
-def get_filename_from_headers(headers: Dict[bytes, List[bytes]]) -> Optional[str]:
+def get_filename_from_headers(headers: dict[bytes, list[bytes]]) -> str | None:
     """
     Get the filename of the downloaded file by inspecting the
     Content-Disposition HTTP header.
@@ -618,7 +615,7 @@ def get_filename_from_headers(headers: Dict[bytes, List[bytes]]) -> Optional[str
     return upload_name
 
 
-def _parse_header(line: bytes) -> Tuple[bytes, Dict[bytes, bytes]]:
+def _parse_header(line: bytes) -> tuple[bytes, dict[bytes, bytes]]:
     """Parse a Content-type like header.
 
     Cargo-culted from `cgi`, but works on bytes rather than strings.
@@ -707,9 +704,9 @@ class ThreadedFileSender:
         self.clock = hs.get_clock()
         self.thread_pool = hs.get_media_sender_thread_pool()
 
-        self.file: Optional[BinaryIO] = None
+        self.file: BinaryIO | None = None
         self.deferred: "Deferred[None]" = Deferred()
-        self.consumer: Optional[interfaces.IConsumer] = None
+        self.consumer: Optional[IConsumer] = None
 
         # Signals if the thread should keep reading/sending data. Set means
         # continue, clear means pause.
@@ -758,9 +755,10 @@ class ThreadedFileSender:
         self.wakeup_event.set()
 
         if not self.deferred.called:
-            self.deferred.errback(
-                ConsumerRequestedStopError("Consumer asked us to stop producing")
-            )
+            with PreserveLoggingContext():
+                self.deferred.errback(
+                    ConsumerRequestedStopError("Consumer asked us to stop producing")
+                )
 
     async def start_read_loop(self) -> None:
         """This is the loop that drives reading/writing"""
@@ -814,7 +812,8 @@ class ThreadedFileSender:
             self.consumer = None
 
         if not self.deferred.called:
-            self.deferred.errback(failure)
+            with PreserveLoggingContext():
+                self.deferred.errback(failure)
 
     def _finish(self) -> None:
         """Called when we have finished writing (either on success or
@@ -828,4 +827,5 @@ class ThreadedFileSender:
             self.consumer = None
 
         if not self.deferred.called:
-            self.deferred.callback(None)
+            with PreserveLoggingContext():
+                self.deferred.callback(None)
