@@ -22,16 +22,9 @@ import logging
 from typing import (
     TYPE_CHECKING,
     Collection,
-    Dict,
-    FrozenSet,
     Iterable,
-    List,
     Mapping,
-    Optional,
     Sequence,
-    Set,
-    Tuple,
-    Union,
     cast,
 )
 
@@ -53,7 +46,7 @@ from synapse.storage.databases.main.stream import (
     generate_pagination_where_clause,
 )
 from synapse.storage.engines import PostgresEngine
-from synapse.types import JsonDict, MultiWriterStreamToken, StreamKeyType, StreamToken
+from synapse.types import JsonDict, StreamKeyType, StreamToken
 from synapse.util.caches.descriptors import cached, cachedList
 
 if TYPE_CHECKING:
@@ -172,14 +165,14 @@ class RelationsWorkerStore(SQLBaseStore):
         room_id: str,
         event_id: str,
         event: EventBase,
-        relation_type: Optional[str] = None,
-        event_type: Optional[str] = None,
+        relation_type: str | None = None,
+        event_type: str | None = None,
         limit: int = 5,
         direction: Direction = Direction.BACKWARDS,
-        from_token: Optional[StreamToken] = None,
-        to_token: Optional[StreamToken] = None,
+        from_token: StreamToken | None = None,
+        to_token: StreamToken | None = None,
         recurse: bool = False,
-    ) -> Tuple[Sequence[_RelatedEvent], Optional[StreamToken]]:
+    ) -> tuple[Sequence[_RelatedEvent], StreamToken | None]:
         """Get a list of relations for an event, ordered by topological ordering.
 
         Args:
@@ -209,7 +202,7 @@ class RelationsWorkerStore(SQLBaseStore):
         assert limit >= 0
 
         where_clause = ["room_id = ?"]
-        where_args: List[Union[str, int]] = [room_id]
+        where_args: list[str | int] = [room_id]
         is_redacted = event.internal_metadata.is_redacted()
 
         if relation_type is not None:
@@ -281,14 +274,14 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def _get_recent_references_for_event_txn(
             txn: LoggingTransaction,
-        ) -> Tuple[List[_RelatedEvent], Optional[StreamToken]]:
+        ) -> tuple[list[_RelatedEvent], StreamToken | None]:
             txn.execute(sql, [event.event_id] + where_args + [limit + 1])
 
             events = []
-            topo_orderings: List[int] = []
-            stream_orderings: List[int] = []
+            topo_orderings: list[int] = []
+            stream_orderings: list[int] = []
             for event_id, relation_type, sender, topo_ordering, stream_ordering in cast(
-                List[Tuple[str, str, str, int, int]], txn
+                list[tuple[str, str, str, int, int]], txn
             ):
                 # Do not include edits for redacted events as they leak event
                 # content.
@@ -316,17 +309,8 @@ class RelationsWorkerStore(SQLBaseStore):
                         StreamKeyType.ROOM, next_key
                     )
                 else:
-                    next_token = StreamToken(
-                        room_key=next_key,
-                        presence_key=0,
-                        typing_key=0,
-                        receipt_key=MultiWriterStreamToken(stream=0),
-                        account_data_key=0,
-                        push_rules_key=0,
-                        to_device_key=0,
-                        device_list_key=MultiWriterStreamToken(stream=0),
-                        groups_key=0,
-                        un_partial_stated_rooms_key=0,
+                    next_token = StreamToken.START.copy_and_replace(
+                        StreamKeyType.ROOM, next_key
                     )
 
             return events[:limit], next_token
@@ -338,8 +322,8 @@ class RelationsWorkerStore(SQLBaseStore):
     async def get_all_relations_for_event_with_types(
         self,
         event_id: str,
-        relation_types: List[str],
-    ) -> List[str]:
+        relation_types: list[str],
+    ) -> list[str]:
         """Get the event IDs of all events that have a relation to the given event with
         one of the given relation types.
 
@@ -354,9 +338,9 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def get_all_relation_ids_for_event_with_types_txn(
             txn: LoggingTransaction,
-        ) -> List[str]:
+        ) -> list[str]:
             rows = cast(
-                List[Tuple[str]],
+                list[tuple[str]],
                 self.db_pool.simple_select_many_txn(
                     txn=txn,
                     table="event_relations",
@@ -377,7 +361,7 @@ class RelationsWorkerStore(SQLBaseStore):
     async def get_all_relations_for_event(
         self,
         event_id: str,
-    ) -> List[str]:
+    ) -> list[str]:
         """Get the event IDs of all events that have a relation to the given event.
 
         Args:
@@ -389,9 +373,9 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def get_all_relation_ids_for_event_txn(
             txn: LoggingTransaction,
-        ) -> List[str]:
+        ) -> list[str]:
             rows = cast(
-                List[Tuple[str]],
+                list[tuple[str]],
                 self.db_pool.simple_select_list_txn(
                     txn=txn,
                     table="event_relations",
@@ -471,13 +455,13 @@ class RelationsWorkerStore(SQLBaseStore):
         return result is not None
 
     @cached()  # type: ignore[synapse-@cached-mutable]
-    async def get_references_for_event(self, event_id: str) -> List[JsonDict]:
+    async def get_references_for_event(self, event_id: str) -> list[JsonDict]:
         raise NotImplementedError()
 
     @cachedList(cached_method_name="get_references_for_event", list_name="event_ids")
     async def get_references_for_events(
         self, event_ids: Collection[str]
-    ) -> Mapping[str, Optional[Sequence[_RelatedEvent]]]:
+    ) -> Mapping[str, Sequence[_RelatedEvent] | None]:
         """Get a list of references to the given events.
 
         Args:
@@ -507,12 +491,12 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def _get_references_for_events_txn(
             txn: LoggingTransaction,
-        ) -> Mapping[str, List[_RelatedEvent]]:
+        ) -> Mapping[str, list[_RelatedEvent]]:
             txn.execute(sql, args)
 
-            result: Dict[str, List[_RelatedEvent]] = {}
+            result: dict[str, list[_RelatedEvent]] = {}
             for relates_to_id, event_id, sender in cast(
-                List[Tuple[str, str, str]], txn
+                list[tuple[str, str, str]], txn
             ):
                 result.setdefault(relates_to_id, []).append(
                     _RelatedEvent(event_id, sender)
@@ -525,14 +509,14 @@ class RelationsWorkerStore(SQLBaseStore):
         )
 
     @cached()  # type: ignore[synapse-@cached-mutable]
-    def get_applicable_edit(self, event_id: str) -> Optional[EventBase]:
+    def get_applicable_edit(self, event_id: str) -> EventBase | None:
         raise NotImplementedError()
 
     # TODO: This returns a mutable object, which is generally bad.
     @cachedList(cached_method_name="get_applicable_edit", list_name="event_ids")  # type: ignore[synapse-@cached-mutable]
     async def get_applicable_edits(
         self, event_ids: Collection[str]
-    ) -> Mapping[str, Optional[EventBase]]:
+    ) -> Mapping[str, EventBase | None]:
         """Get the most recent edit (if any) that has happened for the given
         events.
 
@@ -587,14 +571,14 @@ class RelationsWorkerStore(SQLBaseStore):
                 ORDER by edit.origin_server_ts, edit.event_id
             """
 
-        def _get_applicable_edits_txn(txn: LoggingTransaction) -> Dict[str, str]:
+        def _get_applicable_edits_txn(txn: LoggingTransaction) -> dict[str, str]:
             clause, args = make_in_list_sql_clause(
                 txn.database_engine, "relates_to_id", event_ids
             )
             args.append(RelationTypes.REPLACE)
 
             txn.execute(sql % (clause,), args)
-            return dict(cast(Iterable[Tuple[str, str]], txn.fetchall()))
+            return dict(cast(Iterable[tuple[str, str]], txn.fetchall()))
 
         edit_ids = await self.db_pool.runInteraction(
             "get_applicable_edits", _get_applicable_edits_txn
@@ -612,14 +596,14 @@ class RelationsWorkerStore(SQLBaseStore):
         }
 
     @cached()  # type: ignore[synapse-@cached-mutable]
-    def get_thread_summary(self, event_id: str) -> Optional[Tuple[int, EventBase]]:
+    def get_thread_summary(self, event_id: str) -> tuple[int, EventBase] | None:
         raise NotImplementedError()
 
     # TODO: This returns a mutable object, which is generally bad.
     @cachedList(cached_method_name="get_thread_summary", list_name="event_ids")  # type: ignore[synapse-@cached-mutable]
     async def get_thread_summaries(
         self, event_ids: Collection[str]
-    ) -> Mapping[str, Optional[Tuple[int, EventBase]]]:
+    ) -> Mapping[str, tuple[int, EventBase] | None]:
         """Get the number of threaded replies and the latest reply (if any) for the given events.
 
         Args:
@@ -636,7 +620,7 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def _get_thread_summaries_txn(
             txn: LoggingTransaction,
-        ) -> Tuple[Dict[str, int], Dict[str, str]]:
+        ) -> tuple[dict[str, int], dict[str, str]]:
             # Fetch the count of threaded events and the latest event ID.
             # TODO Should this only allow m.room.message events.
             if isinstance(self.database_engine, PostgresEngine):
@@ -707,7 +691,7 @@ class RelationsWorkerStore(SQLBaseStore):
             args.append(RelationTypes.THREAD)
 
             txn.execute(sql % (clause,), args)
-            counts = dict(cast(List[Tuple[str, int]], txn.fetchall()))
+            counts = dict(cast(list[tuple[str, int]], txn.fetchall()))
 
             return counts, latest_event_ids
 
@@ -735,8 +719,8 @@ class RelationsWorkerStore(SQLBaseStore):
     async def get_threaded_messages_per_user(
         self,
         event_ids: Collection[str],
-        users: FrozenSet[str] = frozenset(),
-    ) -> Dict[Tuple[str, str], int]:
+        users: frozenset[str] = frozenset(),
+    ) -> dict[tuple[str, str], int]:
         """Get the number of threaded replies for a set of users.
 
         This is used, in conjunction with get_thread_summaries, to calculate an
@@ -768,7 +752,7 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def _get_threaded_messages_per_user_txn(
             txn: LoggingTransaction,
-        ) -> Dict[Tuple[str, str], int]:
+        ) -> dict[tuple[str, str], int]:
             users_sql, users_args = make_in_list_sql_clause(
                 self.database_engine, "child.sender", users
             )
@@ -808,7 +792,7 @@ class RelationsWorkerStore(SQLBaseStore):
             user participated in that event's thread, otherwise false.
         """
 
-        def _get_threads_participated_txn(txn: LoggingTransaction) -> Set[str]:
+        def _get_threads_participated_txn(txn: LoggingTransaction) -> set[str]:
             # Fetch whether the requester has participated or not.
             sql = """
                 SELECT DISTINCT relates_to_id
@@ -839,10 +823,10 @@ class RelationsWorkerStore(SQLBaseStore):
 
     async def events_have_relations(
         self,
-        parent_ids: List[str],
-        relation_senders: Optional[List[str]],
-        relation_types: Optional[List[str]],
-    ) -> List[str]:
+        parent_ids: list[str],
+        relation_senders: list[str] | None,
+        relation_types: list[str] | None,
+    ) -> list[str]:
         """Check which events have a relationship from the given senders of the
         given types.
 
@@ -865,8 +849,8 @@ class RelationsWorkerStore(SQLBaseStore):
                 %s;
         """
 
-        def _get_if_events_have_relations(txn: LoggingTransaction) -> List[str]:
-            clauses: List[str] = []
+        def _get_if_events_have_relations(txn: LoggingTransaction) -> list[str]:
+            clauses: list[str] = []
             clause, args = make_in_list_sql_clause(
                 txn.database_engine, "relates_to_id", parent_ids
             )
@@ -944,8 +928,8 @@ class RelationsWorkerStore(SQLBaseStore):
         self,
         room_id: str,
         limit: int = 5,
-        from_token: Optional[ThreadsNextBatch] = None,
-    ) -> Tuple[Sequence[str], Optional[ThreadsNextBatch]]:
+        from_token: ThreadsNextBatch | None = None,
+    ) -> tuple[Sequence[str], ThreadsNextBatch | None]:
         """Get a list of thread IDs, ordered by topological ordering of their
         latest reply.
 
@@ -985,10 +969,10 @@ class RelationsWorkerStore(SQLBaseStore):
 
         def _get_threads_txn(
             txn: LoggingTransaction,
-        ) -> Tuple[List[str], Optional[ThreadsNextBatch]]:
+        ) -> tuple[list[str], ThreadsNextBatch | None]:
             txn.execute(sql, (room_id, *pagination_args, limit + 1))
 
-            rows = cast(List[Tuple[str, int, int]], txn.fetchall())
+            rows = cast(list[tuple[str, int, int]], txn.fetchall())
             thread_ids = [r[0] for r in rows]
 
             # If there are more events, generate the next pagination key from the
